@@ -20,35 +20,46 @@ Contains the definition of a DEP-11 component.
 # You should have received a copy of the GNU Lesser General Public
 # License along with this program.
 
+import os
 import yaml
 import datetime
-from dep11.utils import str_enc_dec, build_cpt_global_id
-from dep11.hints import HintSeverity, hint_tag_is_error
+from .utils import build_cpt_global_id
+from .hints import hint_tag_is_error
 import logging as log
 import hashlib
 
 ###########################################################################
 DEP11_VERSION = "0.8"
-time_str = str(datetime.date.today())
+now = datetime.datetime.now(datetime.timezone.utc).isoformat()
 dep11_header_template = {
     "File": "DEP-11",
-    "Version": DEP11_VERSION
+    "Version": DEP11_VERSION,
+    "Time": now
 }
 ###########################################################################
 
-class DEP11YamlDumper(yaml.Dumper):
-    '''
+class DEP11YamlDumper(yaml.SafeDumper):
+    """
     Custom YAML dumper, to ensure resulting YAML file can be read by
     all parsers (even the Java one)
-    '''
+    """
     def increase_indent(self, flow=False, indentless=False):
         return super(DEP11YamlDumper, self).increase_indent(flow, False)
 
 
-def get_dep11_header(suite_name, component_name, base_url):
+def get_dep11_header(repo_name, suite_name, component_name, base_url, priority):
+    """
+    Build a DEP-11 header YAML document. This document must always be at the start of
+    a valid DEP-11 YAML file.
+    """
     head_dict = dep11_header_template
-    head_dict['Origin'] = "%s-%s" % (suite_name, component_name)
+
+    origin = "%s-%s-%s" % (repo_name, suite_name, component_name)
+    head_dict['Origin'] = origin.lower()
     head_dict['MediaBaseUrl'] = base_url
+    if priority != 0:
+        head_dict['Priority'] = priority
+
     return yaml.dump(head_dict, Dumper=DEP11YamlDumper,
                             default_flow_style=False, explicit_start=True,
                             explicit_end=False, width=200, indent=2)
@@ -79,6 +90,10 @@ class IconSize:
         return "%ix%i" % (self.size, self.size)
 
 
+    def __repr__(self):
+        return "%ix%i" % (self.size, self.size)
+
+
     def __int__(self):
         return self.size
 
@@ -102,11 +117,31 @@ class IconSize:
             return self.size < other.size
         return self.size < other
 
+    def __le__(self, other):
+        if type(other) is IconSize:
+            return self.size <= other.size
+        return self.size < other
+
     def __gt__(self, other):
         if type(other) is IconSize:
             return self.size > other.size
         return self.size > other
 
+    def __ge__(self, other):
+        if type(other) is IconSize:
+            return self.size >= other.size
+        return self.size > other
+
+    def __add__(self, other):
+        if type(other) is IconSize:
+            return self.size + other.size
+        return self.size + other
+
+    def __sub__(self, other):
+        if type(other) is IconSize:
+            return self.size * other.size
+        return self.size * other
+
     def __hash__(self):
         return self.size
 
@@ -119,26 +154,99 @@ class ProvidedItemType:
     BINARY = 'binaries'
     LIBRARY = 'libraries'
     MIMETYPE = 'mimetypes'
-    DBUS = 'dbus'
+    FONT = 'fonts'
+    MODALIAS = 'modaliases'
     PYTHON_2 = 'python2'
     PYTHON_3 = 'python3'
+    DBUS = 'dbus'
     FIRMWARE = 'firmware'
-    CODEC = 'codecs'
 
 
-class DEP11Component:
+class IconType:
+    '''
+    Types an icon can be:
+     * cached: Stored in the AppStream icon cache
+     * stock: Found in an XDG icon theme (which is not hicolor)
+     * remote: On a remote location (usually a webserver)
+    '''
+    CACHED = 'cached'
+    STOCK = 'stock'
+    REMOTE = 'remote'
+
+
+class Screenshot:
+    '''
+    Representation of a DEP-11 screenshot.
+
+    '''
+
+    def __init__(self):
+        self._caption = dict()
+        self._source_img = dict()
+        self._thumbnails = list()
+        self._default = False
+
+
+    def set_source_image(self, url, width, height):
+        self._source_img['width'] = int(width)
+        self._source_img['height'] = int(height)
+        self._source_img['url'] = url
+
+
+    def add_thumbnail(self, url, width, height):
+        thumb = {'width': int(width), 'height': int(height), 'url': url}
+        self._thumbnails.append(thumb)
+
+
+    def has_source_image(self):
+        return self._source_img.get('url') != None
+
+
+    def to_dict(self):
+        d = dict()
+        if self.default:
+            d['default'] = True
+        if self.caption:
+            d['caption'] = self.caption
+        if self._thumbnails:
+            d['thumbnails'] = self._thumbnails
+        d['source-image'] = self._source_img
+        return d
+
+
+    @property
+    def source_image(self):
+        return self._source_img
+
+    @property
+    def caption(self):
+        return self._caption
+
+    @caption.setter
+    def caption(self, val):
+        self._caption = val
+
+    @property
+    def default(self):
+        return self._default
+
+    @default.setter
+    def default(self, val):
+        self._default = bool(val)
+
+
+class Component:
     '''
     Used to store the properties of component data. Used by MetadataExtractor
     '''
 
-    def __init__(self, suitename, component, pkg, pkid=None):
-        '''
-        Used to set the properties to None.
-        '''
+    def __init__(self, suitename, pkg=None):
         self._suitename = suitename
-        self._component = component
-        self._pkg = pkg
-        self._pkid = pkid
+        self._pkid = None
+        self._pkgname = None
+        if pkg:
+            self._pkid = pkg.pkid
+            self._pkgname = pkg.name
 
         # properties
         self._hints = list()
@@ -150,7 +258,7 @@ class DEP11Component:
         self._type = None
         self._name = dict()
         self._categories = None
-        self._icon = None
+        self._icons = dict()
         self._summary = dict()
         self._description = None
         self._screenshots = None
@@ -170,6 +278,8 @@ class DEP11Component:
     def add_hint(self, tag, params=dict()):
         if hint_tag_is_error(tag):
             self._ignore = True
+        if isinstance(params, str):
+            params = {'msg': params}
 
         self._hints.append({'tag': tag, 'params': params})
 
@@ -187,8 +297,8 @@ class DEP11Component:
             hdict['ID'] = self.cid
         if self.kind:
             hdict['Type'] = self.kind
-        if self._pkg:
-            hdict['Package'] = self._pkg
+        if self._pkgname:
+            hdict['Package'] = self._pkgname
         if self._pkid:
             hdict['PackageID'] = self._pkid
         if self.has_ignore_reason():
@@ -210,6 +320,37 @@ class DEP11Component:
         self._srcdata_checksum = md5sum
 
 
+    def set_kind_from_string(self, s):
+        if not s:
+            self.kind = 'generic'
+        elif s == 'desktop':
+            self.kind = 'desktop-app'
+        elif s == 'desktop-app':
+            self.kind = 'desktop-app'
+        elif s == 'font':
+            self.kind = 'font'
+        elif s == 'codec':
+            self.kind = 'codec'
+        elif s == 'inputmethod':
+            self.kind = 'inputmethod'
+        elif s == 'addon':
+            self.kind = 'addon'
+        elif s == 'firmware':
+            self.kind = 'firmware'
+        else:
+            self.add_hint("metainfo-unknown-type", {'type': str(s)})
+
+
+    def build_media_path(self, basepath, subdir):
+        gid = self.global_id
+        if not gid:
+            return None
+        if len(self.cid) < 1:
+            return None
+        path = os.path.join(basepath, gid, subdir)
+        return path
+
+
     @property
     def srcdata_checksum(self):
         return self._srcdata_checksum
@@ -251,7 +392,7 @@ class DEP11Component:
 
     @property
     def pkgname(self):
-        return self._pkg
+        return self._pkgname
 
     @property
     def pkid(self):
@@ -273,14 +414,6 @@ class DEP11Component:
     def categories(self, val):
         self._categories = val
 
-    @property
-    def icon(self):
-        return self._icon
-
-    @icon.setter
-    def icon(self, val):
-        self._icon = val
-
     @property
     def summary(self):
         return self._summary
@@ -392,6 +525,34 @@ class DEP11Component:
         self.provides[kind].append(value)
 
 
+    def get_icon(self, kind):
+        if not self._icons:
+            return self._icons
+        return self._icons.get(kind)
+
+
+    def set_icon(self, kind, value, width=None, height=None):
+        if kind == IconType.REMOTE:
+            self._icons[kind] = dict()
+            self._icons[kind]['width'] = int(width)
+            self._icons[kind]['height'] = int(height)
+            self._icons[kind]['url'] = value
+        else:
+            self._icons[kind] = value
+
+
+    def has_icon(self):
+        if not self._icons:
+            return False
+
+        for key, value in self._icons.items():
+            if value:
+                return True
+
+        self._icons = dict()
+        return False
+
+
     def _is_quoted(self, s):
         return (s.startswith("\"") and s.endswith("\"")) or (s.startswith("\'") and s.endswith("\'"))
 
@@ -410,7 +571,6 @@ class DEP11Component:
 
         unlocalized = d.get('C')
         if unlocalized:
-            to_remove = []
             for k in list(d.keys()):
                 val = d[k]
                 # don't duplicate strings
@@ -447,7 +607,7 @@ class DEP11Component:
         check_for_template(self.developer_name, 'DeveloperName')
         if self.screenshots:
             for i, shot in enumerate(self.screenshots):
-                caption = shot.get('caption')
+                caption = shot.caption
                 if caption:
                     check_for_template(self.developer_name, "Screenshots/%i/caption" % (i))
 
@@ -465,9 +625,8 @@ class DEP11Component:
         self.developer_name = self._cleanup(self.developer_name)
         if self.screenshots:
             for shot in self.screenshots:
-                caption = shot.get('caption')
-                if caption:
-                    shot['caption'] = self._cleanup(caption)
+                if shot.caption:
+                    shot.caption = self._cleanup(shot.caption)
 
         # validate the basics (if we don't ignore this already)
         if not self.has_ignore_reason():
@@ -477,7 +636,7 @@ class DEP11Component:
                 self.add_hint("metainfo-no-type")
             if not self.name:
                 self.add_hint("metainfo-no-name")
-            if not self._pkg:
+            if not self._pkgname:
                 self.add_hint("metainfo-no-package")
             if not self.summary:
                 self.add_hint("metainfo-no-summary")
@@ -485,17 +644,13 @@ class DEP11Component:
             self._check_translated()
 
         d = dict()
-        d['Package'] = str(self._pkg)
+        if self._pkgname:
+            d['Package'] = str(self._pkgname)
         if self.cid:
             d['ID'] = self.cid
         if self.kind:
             d['Type'] = self.kind
 
-        # having the source-data checksum in the final output is useful for
-        # later debugging. It also doesn't use much space.
-        if self.srcdata_checksum:
-            d['X-Source-Checksum'] = self.srcdata_checksum
-
         # check if we need to print ignore information, instead
         # of exporting the software component
         if self.has_ignore_reason():
@@ -513,11 +668,11 @@ class DEP11Component:
         if self.keywords:
             d['Keywords'] = self.keywords
         if self.screenshots:
-            d['Screenshots'] = self.screenshots
+            d['Screenshots'] = list(map(lambda scr: scr.to_dict(), self.screenshots))
         if self.archs:
             d['Architectures'] = self.archs
-        if self.icon:
-            d['Icon'] = {'cached': self.icon}
+        if self._icons:
+            d['Icon'] = self._icons
         if self.url:
             d['Url'] = self.url
         if self.provides:
