@@ -6,7 +6,7 @@ XDG .desktop files.
 """
 
 # Copyright (c) 2014 Abhishek Bhattacharjee <abhishek.bhattacharjee11@gmail.com>
-# Copyright (c) 2014 Matthias Klumpp <mak@debian.org>
+# Copyright (c) 2014-2016 Matthias Klumpp <mak@debian.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -27,10 +27,10 @@ import lxml.etree as et
 from xml.sax.saxutils import escape
 from io import StringIO
 
-from dep11.component import DEP11Component, ProvidedItemType
-from dep11.utils import str_enc_dec
+from .component import Component, Screenshot, IconType, ProvidedItemType
+from .utils import str_enc_dec
 
-def read_desktop_data(cpt, dcontent):
+def read_desktop_data(cpt, dcontent, ignore_nodisplay=False):
     '''
     Parses a .desktop file and sets ComponentData properties
     '''
@@ -40,13 +40,15 @@ def read_desktop_data(cpt, dcontent):
     try:
         df.readfp(StringIO(dcontent))
 
-        items = df.items("Desktop Entry")
-        if df.get("Desktop Entry", "Type") != "Application":
+        dtype = df.get("Desktop Entry", "Type")
+        if dtype and dtype.lower() != "application":
             # ignore this file, isn't an application
             cpt.add_hint("not-an-application")
             return False
+
         try:
-            if df.get("Desktop Entry", "NoDisplay") == "True":
+            nodisplay = df.get("Desktop Entry", "NoDisplay")
+            if not ignore_nodisplay and nodisplay and nodisplay.lower() == "true":
                 # we ignore this .desktop file, shouldn't be displayed
                 cpt.add_hint("invisible-application")
                 return False
@@ -54,13 +56,27 @@ def read_desktop_data(cpt, dcontent):
             # we don't care if the NoDisplay variable doesn't exist
             # if it isn't there, the file should be processed
             pass
+
+        try:
+            asignore = df.get("Desktop Entry", "X-AppStream-Ignore")
+            if asignore and asignore.lower() == "true":
+                # this .desktop file should be excluded from AppStream metadata
+                cpt.add_hint("invisible-application")
+                return False
+        except:
+            # we don't care if the AppStream-Ignore variable doesn't exist
+            # if it isn't there, the file should be processed
+            pass
+
+        items = df.items('Desktop Entry')
+
     except Exception as e:
         # this .desktop file is not interesting
         cpt.add_hint("desktop-file-read-error", str(e))
         return True
 
     # if we reached this step, we are dealing with a GUI desktop app
-    cpt.kind = 'desktop-app'
+    cpt.set_kind_from_string('desktop-app')
 
     for item in items:
         if len(item) != 2:
@@ -69,6 +85,8 @@ def read_desktop_data(cpt, dcontent):
         value = str_enc_dec(item[1])
         if not value:
             continue
+        value = value.strip()
+
         if key.startswith("name"):
             if key == 'name':
                 cpt.name['C'] = value
@@ -116,11 +134,9 @@ def read_desktop_data(cpt, dcontent):
             if len(value) > 1:
                 value.pop()
             for val in value:
-                cpt.add_provided_item(
-                    ProvidedItemType.MIMETYPE, val
-                )
+                cpt.add_provided_item(ProvidedItemType.MIMETYPE, val)
         elif key == 'icon':
-            cpt.icon = value
+            cpt.set_icon(IconType.CACHED, value)
     return True
 
 
@@ -191,17 +207,18 @@ def _parse_screenshots_tag(subs):
     for usubs in subs:
         # for one screeshot tag
         if usubs.tag == 'screenshot':
-            screenshot = dict()
+            shot = Screenshot()
             attr_dic = usubs.attrib
             if attr_dic.get('type'):
                 if attr_dic['type'] == 'default':
-                    screenshot['default'] = True
-            # in case of old styled xmls
-            url = usubs.text
+                    shot.default = True
+
+            # handle pre-0.6 spec screenshot notations
+            url = usubs.text.strip() if usubs.text else None
             if url:
-                url = url.strip()
-                screenshot['source-image'] = {'url': url}
-                shots.append(screenshot)
+                # we do not know width or height yet, that information will be added later
+                shot.set_source_image(url, 0, 0)
+                shots.append(shot)
                 continue
 
             # else look for captions and image tag
@@ -215,16 +232,15 @@ def _parse_screenshots_tag(subs):
                     else:
                         key = 'C'
 
-                    if screenshot.get('caption'):
-                        screenshot['caption'][key] = str_enc_dec(tags.text)
-                    else:
-                        screenshot['caption'] = {key: str_enc_dec(tags.text)}
+                    caption_text = str_enc_dec(tags.text)
+                    if caption_text:
+                        shot.caption[key] = caption_text
                 if tags.tag == 'image':
-                    screenshot['source-image'] = {'url': tags.text}
+                    shot.set_source_image(tags.text, 0, 0)
 
             # only add the screenshot if we have a source image
-            if screenshot.get ('source-image'):
-                shots.append(screenshot)
+            if shot.has_source_image():
+                shots.append(shot)
 
     return shots
 
@@ -274,11 +290,15 @@ def _parse_releases_tag(relstag):
 
 def read_appstream_upstream_xml(cpt, xml_content):
     '''
-    Reads the appdata from the xml file in usr/share/appdata.
+    Reads the metadata from the xml file in usr/share/metainfo.
     Sets ComponentData properties
     '''
+
     root = None
     try:
+        # Drop default namespace - some add a bogus namespace to their metainfo files which breaks the parser.
+        # When we actually start using namespaces in future, we need to handle them explicitly.
+        xml_content = re.sub(r'\sxmlns="[^"]+"', '', xml_content, count=1)
         root = et.fromstring(bytes(xml_content, 'utf-8'))
     except Exception as e:
         cpt.add_hint("metainfo-parse-error", str(e))
@@ -291,32 +311,28 @@ def read_appstream_upstream_xml(cpt, xml_content):
         # oddities and to simplify the parser in future. So we add a hint for that.
         cpt.add_hint("ancient-metadata")
 
-    key = root.attrib.get('type')
-    if key:
-        if key == 'desktop':
-            cpt.kind = 'desktop-app'
-        else:
-            # for other components like addon,codec, inputmethod etc
-            cpt.kind = root.attrib['type']
+    # set the type of our component
+    cpt.set_kind_from_string(root.attrib.get('type'))
 
     for subs in root:
         locale = _get_tag_locale(subs)
 
+        value = None
+        if subs.text:
+            value = subs.text.strip()
+
         if subs.tag == 'id':
-            cpt.cid = subs.text
-            # legacy support
-            key = subs.attrib.get('type')
-            if key and not cpt.kind:
-                if key == 'desktop':
-                    cpt.kind = 'desktop-app'
-                else:
-                    cpt.kind = key
+            cpt.cid = value
+            # INFO: legacy support, remove later
+            tps = subs.attrib.get('type')
+            if tps:
+                cpt.set_kind_from_string(tps)
 
         elif subs.tag == "name":
-            cpt.name[locale] = subs.text
+            cpt.name[locale] = value
 
         elif subs.tag == "summary":
-            cpt.summary[locale] = subs.text
+            cpt.summary[locale] = value
 
         elif subs.tag == "description":
             desc = _parse_description_tag(subs)
@@ -327,74 +343,84 @@ def read_appstream_upstream_xml(cpt, xml_content):
             cpt.screenshots = screen
 
         elif subs.tag == "provides":
-            for bins in subs:
-                if bins.tag == "binary":
+            for ptag in subs:
+                ptag_text = None
+                if ptag.text:
+                    ptag_text = ptag.text.strip()
+                if ptag.tag == "binary":
                     cpt.add_provided_item(
-                        ProvidedItemType.BINARY, bins.text
+                        ProvidedItemType.BINARY, ptag_text
                     )
-                if bins.tag == 'library':
+                if ptag.tag == 'library':
                     cpt.add_provided_item(
-                        ProvidedItemType.LIBRARY, bins.text
+                        ProvidedItemType.LIBRARY, ptag_text
                     )
-                if bins.tag == 'dbus':
-                    if not cpt.provides.get(ProvidedItemType.DBUS):
-                        cpt.provides[ProvidedItemType.DBUS] = list()
-
-                    bus_kind = bins.attrib.get('type')
+                if ptag.tag == 'dbus':
+                    bus_kind = ptag.attrib.get('type')
                     if bus_kind == "session":
                         bus_kind = "user"
                     if bus_kind:
-                        cpt.provides[ProvidedItemType.DBUS].append({'type': bus_kind, 'service': bins.text})
-                if bins.tag == 'firmware':
-                    if not cpt.provides.get(ProvidedItemType.FIRMWARE):
-                        cpt.provides[ProvidedItemType.FIRMWARE] = list()
-
-                    fw_type = bins.attrib.get('type')
+                        cpt.add_provided_item(ProvidedItemType.DBUS, {'type': bus_kind, 'service': ptag_text})
+                if ptag.tag == 'firmware':
+                    fw_type = ptag.attrib.get('type')
                     fw_data = {'type': fw_type}
 
-                    _valid = True
+                    fw_valid = True
                     if fw_type == "flashed":
-                        fw_data['guid'] = bins.text
+                        fw_data['guid'] = ptag_text
                     elif fw_type == "runtime":
-                        fw_data['fname'] = bins.text
+                        fw_data['fname'] = ptag_text
                     else:
-                        _valid = False
+                        fw_valid = False
 
-                    if _valid:
-                        cpt.provides[ProvidedItemType.FIRMWARE].append(fw_data)
-                if bins.tag == 'python2':
+                    if fw_valid:
+                        cpt.add_provided_item(ProvidedItemType.FIRMWARE, fw_data)
+                if ptag.tag == 'python2':
                     cpt.add_provided_item(
-                        ProvidedItemType.PYTHON_2, bins.text
+                        ProvidedItemType.PYTHON_2, ptag_text
                     )
-                if bins.tag == 'python3':
+                if ptag.tag == 'python3':
                     cpt.add_provided_item(
-                        ProvidedItemType.PYTHON_3, bins.text
+                        ProvidedItemType.PYTHON_3, ptag_text
                     )
-                if bins.tag == 'codec':
+                if ptag.tag == 'modalias':
                     cpt.add_provided_item(
-                        ProvidedItemType.CODEC, bins.text
+                        ProvidedItemType.MODALIAS, ptag_text
+                    )
+                if ptag.tag == 'mimetype':
+                    cpt.add_provided_item(
+                        ProvidedItemType.MIMETYPE, ptag_text
+                    )
+                if ptag.tag == 'font':
+                    font_file = ptag.attrib.get('file')
+                    if font_file:
+                        cpt.add_provided_item(ProvidedItemType.FONT, {'file': font_file, 'name': ptag_text})
+        elif subs.tag == "mimetypes":
+            for mimetag in subs:
+                if mimetag.tag == "mimetype":
+                    cpt.add_provided_item(
+                        ProvidedItemType.MIMETYPE, mimetag.text
                     )
-
         elif subs.tag == "url":
             if cpt.url:
-                cpt.url.update({subs.attrib['type']: subs.text})
+                cpt.url.update({subs.attrib['type']: value})
             else:
-                cpt.url = {subs.attrib['type']: subs.text}
+                cpt.url = {subs.attrib['type']: value}
 
         elif subs.tag == "project_license":
-            cpt.project_license = subs.text
+            cpt.project_license = value
 
         elif subs.tag == "project_group":
-            cpt.project_group = subs.text
+            cpt.project_group = value
 
         elif subs.tag == "developer_name":
-            cpt.developer_name[locale] = subs.text
+            cpt.developer_name[locale] = value
 
         elif subs.tag == "extends":
-            cpt.extends.append(subs.text)
+            cpt.extends.append(value)
 
         elif subs.tag == "compulsory_for_desktop":
-            cpt.compulsory_for_desktops.append(subs.text)
+            cpt.compulsory_for_desktops.append(value)
 
         elif subs.tag == "releases":
             releases = _parse_releases_tag(subs)
