@@ -17,6 +17,8 @@
 
 import yaml
 import gzip
+from optparse import OptionParser
+import sys
 import xml.etree.ElementTree as ET
 from voluptuous import Schema, Required, All, Any, Length, Range, Match, Url
 
@@ -26,9 +28,10 @@ schema_header = Schema({
     Required('File'): All(str, 'DEP-11', msg="Must be \"DEP-11\""),
     Required('Origin'): All(str, Length(min=1)),
     Required('Version'): All(str, Match(r'(\d+\.?)+$'), msg="Must be a valid version number"),
-    Required('MediaBaseUrl'): All(str, Url()),
-    'Time': All(str, str),
-    'Priority': All(str, int),
+    Required('MediaBaseUrl'): All(Url()),
+    'Time': All(str),
+    'Priority': All(int),
+    'Architecture': All(str),
 })
 
 schema_provides_dbus = Schema({
@@ -41,14 +44,19 @@ schema_provides_firmware = Schema({
     Any('guid', 'fname'): All(str, Length(min=1))
 })
 
+schema_provides_font = Schema({
+    Required('file'): All(str, Length(min=1)),
+    Required('name'): All(str, Length(min=1)),
+})
+
 schema_provides = Schema({
     Any('mimetypes',
         'binaries',
         'libraries',
         'python3',
         'python2',
-        'modaliases',
-        'fonts'): All(list, [str], Length(min=1)),
+        'modaliases'): All(list, [str], Length(min=1)),
+        'fonts': All(list, Length(min=1), [schema_provides_font]),
         'dbus': All(list, Length(min=1), [schema_provides_dbus]),
         'firmware': All(list, Length(min=1), [schema_provides_firmware]),
 })
@@ -66,7 +74,7 @@ schema_translated = Schema({
 schema_image = Schema({
     Required('width'): All(int, Range(min=10)),
     Required('height'): All(int, Range(min=10)),
-    Required('url'): All(str, str, Length(min=1)),
+    Required('url'): All(str, Length(min=1)),
 })
 
 schema_screenshots = Schema({
@@ -76,11 +84,17 @@ schema_screenshots = Schema({
     'caption': All(dict, Length(min=1), schema_translated),
 })
 
+schema_icon_local = Schema({
+    Required('width'): All(int, Range(min=10)),
+    Required('height'): All(int, Range(min=10)),
+    Required('path'): All(str, Match(r'^[\'"]?(?:/[^/]+)*[\'"]?$'), msg='Local icon entry must be an absolute path'),
+})
+
 schema_icon = Schema({
     'stock': All(str, Length(min=1)),
     'cached': All(str, Match(r'.*[.].*$'), msg='Icon entry is missing filename or extension'),
-    'local': All(str, Match(r'^[\'"]?(?:/[^/]+)*[\'"]?$'), msg='Icon entry should be an absolute path'),
-    'remote': All(str, str, Length(min=1)),
+    'local': All(dict, Length(min=1), schema_icon_local),
+    'remote': All(dict, Length(min=1), schema_image),
 })
 
 schema_url = Schema({
@@ -98,7 +112,7 @@ schema_releases = Schema({
 })
 
 schema_component = Schema({
-    Required('Type'): All(str, Any('generic', 'desktop-app', 'web-app', 'addon', 'codec', 'inputmethod', 'font')),
+    Required('Type'): All(str, Any('generic', 'desktop-app', 'web-app', 'addon', 'codec', 'inputmethod', 'font', 'firmware')),
     Required('ID'): All(str, Length(min=1)),
     Required('Name'): All(dict, Length(min=1), schema_translated),
     Required('Package'): All(str, Length(min=1)),
@@ -116,9 +130,6 @@ schema_component = Schema({
     'Screenshots': All(list, Length(min=1), [schema_screenshots]),
     'Extends': All(list, [str], Length(min=1)),
     'Releases': All(list, Length(min=1), [schema_releases]),
-
-    # Internal, non-specified fields
-    'X-Source-Checksum': All(str, Length(min=10)),
 })
 
 class DEP11Validator:
@@ -205,7 +216,7 @@ class DEP11Validator:
         ret = self._test_custom_objects(lines)
 
         try:
-            docs = yaml.load_all(data)
+            docs = yaml.safe_load_all(data)
             header = next(docs)
         except Exception as e:
             self.add_issue("Could not parse file: %s" % (str(e)))
@@ -245,7 +256,7 @@ class DEP11Validator:
 
             # more tests for the icon key
             icon = doc.get('Icon')
-            if (doc['Type'] == "desktop-app") or (doc['Type'] == "web-app"):
+            if (doc['Type'] == 'desktop-app') or (doc['Type'] == 'web-app'):
                 if not doc.get('Icon'):
                     self.add_issue("[%s]: %s" % (docid, "Components containing an application must have an 'Icon' key."))
                     ret = False
@@ -291,7 +302,7 @@ class DEP11Validator:
         if fname.endswith(".gz"):
             f = gzip.open(fname, 'r')
         else:
-            f = open(fname, 'r')
+            f = open(fname, 'rb')
 
         data = str(f.read(), 'utf-8')
         f.close()
@@ -306,3 +317,35 @@ class DEP11Validator:
         self.issue_list = list()
 
 __all__.append('DEP11Validator')
+
+def main():
+    """Main entry point of validator"""
+
+    parser = OptionParser()
+    parser.add_option("--no-color",
+                  action="store_true", dest="no_color", default=False,
+                  help="don't print colored output")
+
+    (options, args) = parser.parse_args()
+
+    if len(args) < 1:
+        print("You need to specify a file to validate!")
+        sys.exit(4)
+    fname = args[0]
+
+    validator = DEP11Validator()
+    ret = validator.validate_file(fname)
+    validator.print_issues()
+    if ret:
+        msg = "Validation successful."
+    else:
+        msg = "Validation failed!"
+    if options.no_color:
+        print(msg)
+    elif ret:
+        print('\033[92m' + msg + '\033[0m')
+    else:
+        print('\033[91m' + msg + '\033[0m')
+
+    if not ret:
+        sys.exit(1)
